/*
========================================
HP 2100M P(rinter) J(ob) L(anguage) Tool
========================================
May be compatible with other hp ljet printers

Copyright (C) 1999 Bob Duke
bduke@users.sourceforge.net

Check out kljettool if you have an older LaserJet
Visit http://www.kde.org/ for more details

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<string.h>

/* Number of characters allowed in firmware */
#define PRINTER_NAME_MAX_LENGTH 32 

/* Every PJL sequence must start (and end) with a specific escape sequence: */
/* escape code pulled from kljettool source */
static char escape_sequence[4] = "\x1b\x25\x2d"; 

/* Function prototypes */
int check_length_printername(char *name, int max_length);
void print_help(char *argv[]);

int verbose = 0; /* set to one to enable verbose output */

int main(int argc, char *argv[]){
  
  FILE *pipe=NULL;
  char lpr[30]="lpr -P";
  char *pipe_command;
  char *lpr_printer_name=NULL; /* name of printer used for lpr pipe */

  char *printer_name=NULL; /* name of printer in firmware */
  char temp_name[PRINTER_NAME_MAX_LENGTH+1]; 

  char *economode=NULL; 
  
  char *darkcourier = NULL;
 

  int density=0; /* can be 1,2,3,4 or 5 */
  int resolution=0; /* can be 300,600, 1200 */
  int testpage=0; /* can be 1,2,3,4,5 */ 

  int lock_tray=0; /* can be 1,2, or 3... 3 only valid if optional
		      tray three is installed */
  int unlock_tray = 0;

  int  opt=0;

  if(argc < 2){
    /* no options, print help */
    print_help(argv);
  }

  /* Check to see what options were passed: */
  while((opt = getopt(argc,argv,"C:D:E:hP:R:t:vN:U:L:"))!=-1){
    switch(opt){  
    case 'C':
      darkcourier = optarg;
      break;
    case 'D':
      density = atoi(optarg); 
      break;
    case 'E':
      economode = optarg;
      break;
    case 'h':
      print_help(argv);
      break;
    case 'L':
      lock_tray = atoi(optarg);
    case 'P':
      lpr_printer_name = optarg;
      break;
    case 'R':
      resolution = atoi(optarg);
      break;
    case 't':
      testpage = atoi(optarg);
      break;
    case 'v':
      verbose = 1;
      printf("You have selected verbose mode. Good Choice.\n");
      break;
    case 'U':
      unlock_tray = atoi(optarg);
      break;
    case 'N':
      printf("Guess What? The 2100 doesn't support the -N option :-(\n");
      if(check_length_printername(optarg, PRINTER_NAME_MAX_LENGTH)){
	printer_name = optarg;
      }else{
	printf("Printer Name can be no more than 32 characters!\n");
	strncpy(temp_name,optarg,32);
	temp_name[32]='\0'; /* add the null zero to terminate the string */
        printer_name = temp_name;
      }
      break;
    default:
      break;
    }
  }

  if(verbose){
    printf("lpr printer name = %s\n", lpr_printer_name);
    printf("printer name = %s\n", printer_name);
    printf("economode = %s\n",economode);
    printf("density = %d\n", density);
  }


  /****************************/
  /* Getting the printer name */
  /****************************/
  if(lpr_printer_name == NULL){
    if(verbose){
      printf("Printer Name not passed.. looking at PRINTER env variable...\n");
    }
    lpr_printer_name = getenv("PRINTER");
  }
  
  /* If lpr_printer_name is still NULL, we're out of luck */
  if(lpr_printer_name == NULL){
    printf("Cannot open a printer. Specify valid printer with -P option\n");
    exit(EXIT_FAILURE);
  }
  
  if(strlen(lpr_printer_name)+1 > 23){
    printf("Printer name too long (i.e. this should be coded better, fix later)\n");
  }
  /*********************************/
  /* End Getting the printer name */
  /********************************/
  


  strcat(lpr,lpr_printer_name);
  pipe = popen(lpr,"w");


  if(pipe==NULL){
    printf("error opening pipe to lpr\n");
    exit(EXIT_FAILURE);
  }

  /* PJL Initialize -- let's the printer know some 
     commands are coming... */
  fwrite(escape_sequence,3,1,pipe);
  fprintf(pipe,"12345X@PJL\n");
  /* end header, begin commands (self-explanatory) */
  /* there MUST be a newline at the end of each command */


  /* Omit... 2100 does not support the PRINTERNAME option 
  if(printer_name != NULL){
    fprintf(pipe,"@PJL DEFAULT PRINTERNAME = %s\n", printer_name);
  }
  */

  /***************************/
  /* Send ECONOMODE commands */
  /***************************/

  if(economode != NULL){
    if(strcmp(economode,"on")==0){
      /* the HP2100 will NOT support economode if set at 1200 dpi */
      if(verbose){
	printf("@PJL DEFAULT RESOLUTION = %s\n", "600");
	printf("@PJL DEFAULT ECONOMODE = %s\n",  "ON"); 
      }
      fprintf(pipe,"@PJL DEFAULT RESOLUTION = %s\n", "600");
      fprintf(pipe,"@PJL DEFAULT ECONOMODE = %s\n",  "ON"); 
    }else if(strcmp(economode,"off")==0){
      if(verbose){
	printf("@PJL DEFAULT ECONOMODE = %s\n",	"OFF");
	printf("@PJL DEFAULT RESOLUTION = %s\n", "1200");
      }
      fprintf(pipe,"@PJL DEFAULT ECONOMODE = %s\n",	"OFF");
      /* set the printer back to 1200 */
      fprintf(pipe,"@PJL DEFAULT RESOLUTION = %s\n", "1200");
    }else{
      printf("invalid option for -E option: %s\n",economode);
      printf("only on or off is allowed\n");
    }
  }


  /****************************/
  /* Send RESOLUTION commands */
  /****************************/

   if(resolution != 0){
    if((resolution == 300) || (resolution == 600) || (resolution == 1200) ){
      if(verbose) printf("Sending @PJL DEFAULT RESOLUTION=%d\n", resolution);
      fprintf(pipe,"@PJL DEFAULT RESOLUTION=%d\n", resolution);
    }else{
      printf("invalid option for -R option: %d\n", resolution);
      printf("only 300,600, or 1200 is allowed\n");
    }
  }

  /*************************/
  /* Send DENSITY commands */
  /*************************/
  
  if(density != 0){
    if((density <= 5) && (density >= 1)){

      if(verbose) printf("Sending @PJL DEFAULT DENSITY = %d\n", density);
      fprintf(pipe,"@PJL DEFAULT DENSITY = %d\n", density);

    }else{
      printf("invalid option for -D option: %d\n", density);
      printf("only 1,2,3,4, or 5 is allowed\n");
    }
  }

  /*****************************/
  /* Send TRAY_LOCK commands   */
  /*****************************/
  
  if(lock_tray != 0){
    if(lock_tray == 1){
      if(verbose) printf("Sending @PJL DEFAULT INTRAY1 = LOCKED\n");
      fprintf(pipe,"@PJL DEFAULT INTRAY1 = LOCKED\n");
    }else
    if(lock_tray == 2){
      if(verbose) printf("Sending @PJL DEFAULT INTRAY2 = LOCKED\n");
      fprintf(pipe,"@PJL DEFAULT INTRAY2 = LOCKED\n");
    }else
    if(lock_tray == 3){
      if(verbose) printf("Sending @PJL DEFAULT INTRAY3 = LOCKED\n");
      fprintf(pipe,"@PJL DEFAULT INTRAY3 = LOCKED\n");
    }else{
      printf("invalid option for -L option: %d\n", lock_tray);
      printf("only 1,2, or 3 is allowed\n");
    }
  }

  /******************************/
  /* Send TRAY_UNLOCK commands  */
  /******************************/
  
  if(unlock_tray != 0){
    if(unlock_tray == 1){
      if(verbose) printf("Sending @PJL DEFAULT INTRAY1 = UNLOCKED\n");
      fprintf(pipe,"@PJL DEFAULT INTRAY1 = UNLOCKED\n");
    }else
    if(unlock_tray == 2){
      if(verbose) printf("Sending @PJL DEFAULT INTRAY2 = UNLOCKED\n");
      fprintf(pipe,"@PJL DEFAULT INTRAY2 = UNLOCKED\n");
    }else
    if(unlock_tray == 3){
      if(verbose) printf("Sending @PJL DEFAULT INTRAY3 = UNLOCKED\n");
      fprintf(pipe,"@PJL DEFAULT INTRAY3 = UNLOCKED\n");
    }else{
      printf("invalid option for -U option: %d\n", unlock_tray);
      printf("only 1,2, or 3 is allowed\n");
    }
  }

  /**************************/
  /* Send TESTPAGE commands */
  /**************************/
  if(testpage != 0){
    if((testpage <= 5) && (testpage >= 1)){
      switch(testpage){
        case 1:
	  /* Print the configuration/self test page */ 
	  if(verbose) printf("@PJL DEFAULT TESTPAGE=SELFTEST\n"); 
	  fprintf(pipe,"@PJL DEFAULT TESTPAGE=SELFTEST\n");
	  break;
        case 2:
	  if(verbose) printf("@PJL DEFAULT TESTPAGE=PCLTYPELIST\n"); 
	  fprintf(pipe,"@PJL DEFAULT TESTPAGE=PCLTYPELIST\n");
	  break;
        case 3:
	  if(verbose) printf("@PJL DEFAULT TESTPAGE=PSTYPEFACELIST\n"); 
	  fprintf(pipe,"@PJL DEFAULT TESTPAGE=PSTYPEFACELIST\n");
	  break;
	case 4:
	  /* In PJL spec, but the 2100 doesn't respond */
	  if(verbose) printf("@PJL DEFAULT TESTPAGE=PSCONFIGPAGE\n"); 
	  fprintf(pipe,"@PJL DEFAULT TESTPAGE=PSCONFIGPAGE\n");
	  break;
        case 5:
	  /* In PJL spec, but the 2100 doesn't respond */
	  if(verbose) printf("@PJL DEFAULT TESTPAGE=PSDEMOPAGE\n"); 
	  fprintf(pipe,"@PJL DEFAULT PRTPSERRS=ON\n");
	  fprintf(pipe,"@PJL DEFAULT TESTPAGE=PSDEMOPAGE\n");
	  break;
        default:
	  break;
      }
    }else{
      printf("invalid option for -t option: %d\n", testpage);
      printf("only 1,2,3,4, or 5 is allowed\n");
    }
  }

  /*************************/
  /* Send COURIER commands */
  /*************************/
  
  if(darkcourier != NULL){

    if(strcmp(darkcourier,"on")==0){

      /* Turn on Dark Courier */
      if(verbose) printf("@PJL DEFAULT COURIER=DARK\n"); 
      fprintf(pipe,"@PJL DEFAULT COURIER=DARK\n"); 

    }else if(strcmp(darkcourier,"off")==0){

      /* Turn off Dark Courier */
      if(verbose) printf("@PJL DEFAULT COURIER=REGULAR\n"); 
      fprintf(pipe,"@PJL DEFAULT COURIER=REGULAR\n"); 

    }else{
      printf("invalid option for -C option: %s\n", darkcourier);
      printf("only on or off is allowed\n");
    }
  }
  
  
  /* Tell the printer that no more commands are coming... */
  fwrite(escape_sequence,3,1,pipe); 
  fprintf(pipe,"12345X\n");
  
  pclose(pipe);

  exit(EXIT_SUCCESS);
}

/* returns 1 if *name has a length less than max_length, 0 otherwise */
int check_length_printername(char *name, int max_length){
  
  if(strlen(name) > max_length){
    return 0;
  }
  else{
    return 1;
  }
}


void print_help(char *argv[]){
  printf("usage: %s [-Pprinter] [-hv] [-D num -E on|off -L 1|2|3 -U 1|2|3 -R 300|600|1200 -t num]\n",argv[0]);
  printf("Uses PRINTER env variable if -P option is not specified\n");
  exit(EXIT_SUCCESS);
}










