/*
g2100config - callbacks.c

Copyright (C) 2000 Bob Duke

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "struct.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define PRINTER_NAME_MAX_LENGTH 32 

struct printer_data printer_data = {NULL, FALSE, 3, FALSE, 1200, 0, 0, 0};
gint testpage_num = 1; /* Keeps track of the desired testpage */
gint verbose = 0; /* set to 1 for diagnostic output */
static char escape_sequence[4] = "\x1b\x25\x2d"; 

int check_length_printername(char *name, int max_length);

gint
send_data_to_printer                   (GtkButton       *button,
                                        gpointer         user_data)
{
   FILE *pipe=NULL;
   char lpr[60]="lpr -P";
   char temp_name[PRINTER_NAME_MAX_LENGTH+1]; 

   if (verbose)
   {
      g_print ("=======================\n");
      g_print  ("Printer Name: %s\n", printer_data.name);
      g_print ("=======================\n");
      if (printer_data.one_lock) g_print ("Lock Tray One\n");
      if (printer_data.two_lock) g_print ("Lock Tray Two\n");
      if (printer_data.three_lock) g_print ("Lock Tray Three\n");
   }

   if (check_length_printername (printer_data.name, PRINTER_NAME_MAX_LENGTH))
   { 
      /* printer name is not too long, do nothing*/
   }
   else
   {
      g_print ("Printer Name can be no more than 32 characters\n");
      g_print ("(yes, this is lame, and will eventually be fixed...)\n");
      g_print ("For now, the name will be truncated...\n");
      strncpy (temp_name, printer_data.name, 32);
      temp_name[32] = '\0'; /* add ASCII null zero to terminate string */
      printer_data.name = temp_name;
   }
		   

   strcat (lpr,printer_data.name);
   pipe = popen(lpr,"w");


   if(pipe==NULL){
      g_print ("error opening pipe to lpr\n");
      return (1);
   }

   /* PJL Initialize -- let's the printer know some 
     commands are coming... */
   fwrite (escape_sequence,3,1,pipe);
   fprintf (pipe,"12345X@PJL\n");
   /* end header, begin commands (self-explanatory) */
   /* there MUST be a newline at the end of each command */

  /***************************/
  /* Send ECONOMODE commands */
  /***************************/

   if (printer_data.economode == 0)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT ECONOMODE = %s\n", "OFF");
      fprintf (pipe,"@PJL DEFAULT ECONOMODE = OFF\n");
   }

   if (printer_data.economode == 1)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT ECONOMODE = %s\n", "ON");
      fprintf (pipe,"@PJL DEFAULT ECONOMODE = ON\n");
   }

   /****************************/
   /* Send RESOLUTION commands */
   /****************************/

   if(verbose) printf("Sending @PJL DEFAULT RESOLUTION = %d\n", printer_data.resolution);
   fprintf(pipe,"@PJL DEFAULT RESOLUTION=%d\n", printer_data.resolution);
  
  /*************************/
  /* Send DENSITY commands */
  /*************************/
  
   if(verbose) printf("Sending @PJL DEFAULT DENSITY = %d\n", printer_data.density);
   fprintf(pipe,"@PJL DEFAULT DENSITY = %d\n", printer_data.density);
  
  
  /*****************************/
  /* Send TRAY_LOCK commands   */
  /*****************************/
  
   if(printer_data.one_lock == 1)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT INTRAY1 = LOCKED\n");
      fprintf (pipe,"@PJL DEFAULT INTRAY1 = LOCKED\n");
   }
  
   if(printer_data.two_lock == 1)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT INTRAY2 = LOCKED\n");
      fprintf (pipe,"@PJL DEFAULT INTRAY2 = LOCKED\n");
   }
  
   if(printer_data.three_lock == 1)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT INTRAY3 = LOCKED\n");
      fprintf (pipe,"@PJL DEFAULT INTRAY3 = LOCKED\n");
   }
  
   /******************************/
   /* Send TRAY_UNLOCK commands  */
   /******************************/
   if(printer_data.one_lock == 0)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT INTRAY1 = UNLOCKED\n");
      fprintf (pipe,"@PJL DEFAULT INTRAY1 = UNLOCKED\n");
   }
  
   if(printer_data.two_lock == 0)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT INTRAY2 = UNLOCKED\n");
      fprintf (pipe,"@PJL DEFAULT INTRAY2 = UNLOCKED\n");
   }
  
   if(printer_data.three_lock == 0)
   {
      if (verbose) g_print ("Sending @PJL DEFAULT INTRAY3 = UNLOCKED\n");
      fprintf (pipe,"@PJL DEFAULT INTRAY3 = UNLOCKED\n");
   }

   /*************************/
   /* Send COURIER commands */
   /*************************/

   if (printer_data.darkcourier == 0)
   {
      /* Turn off Dark Courier */
      if(verbose) printf("Sending @PJL DEFAULT COURIER = REGULAR\n"); 
      fprintf(pipe,"@PJL DEFAULT COURIER=REGULAR\n"); 
   }

   if (printer_data.darkcourier == 1)
   {
      /* Turn on Dark Courier */
      if(verbose) printf("Sending @PJL DEFAULT COURIER = DARK\n"); 
      fprintf(pipe,"@PJL DEFAULT COURIER=DARK\n"); 
   }
  
   /* Tell the printer that no more commands are coming... */
   fwrite(escape_sequence,3,1,pipe); 
   fprintf(pipe,"12345X\n");
  
   pclose(pipe);

   if (atoi(user_data))
   {
      /* pressed ok button, so quit */
       gtk_main_quit();
       return(FALSE);
   }else
   {
      /* pressed apply, so DON'T QUIT */
      return(0);
   }
}


void
print_testpage                         (GtkButton       *button,
                                        gpointer         user_data)
{
   FILE *pipe=NULL;
   char lpr[60]="lpr -P";
   char temp_name[PRINTER_NAME_MAX_LENGTH+1];
   
   if (check_length_printername (printer_data.name, PRINTER_NAME_MAX_LENGTH))
   { 
      /* printer name is not too long, do nothing*/
   }
   else
   {
      g_print ("Printer Name can be no more than 32 characters\n");
      g_print ("(yes, this is lame, and will eventually be fixed...)\n");
      g_print ("For now, the name will be truncated...\n");
      strncpy (temp_name, printer_data.name, 32);
      temp_name[32] = '\0'; /* add ASCII null zero to terminate string */
      printer_data.name = temp_name;
   }
   
   
   strcat (lpr,printer_data.name);
   pipe = popen(lpr,"w");
   
   
   if(pipe==NULL){
      g_print ("error opening pipe to lpr\n");
      return;
   }
   
   /* PJL Initialize -- let's the printer know some 
     commands are coming... */
   fwrite (escape_sequence,3,1,pipe);
   fprintf (pipe,"12345X@PJL\n");
   /* end header, begin commands (self-explanatory) */
   /* there MUST be a newline at the end of each command */
   
   switch(testpage_num){
      case 1:
	 /* Print the configuration/self test page */ 
	 if(verbose) printf("Sending @PJL DEFAULT TESTPAGE=SELFTEST\n"); 
	 fprintf(pipe,"@PJL DEFAULT TESTPAGE=SELFTEST\n");
	 break;
      case 2:
	 if(verbose) printf("Sending @PJL DEFAULT TESTPAGE=PCLTYPELIST\n"); 
	 fprintf(pipe,"@PJL DEFAULT TESTPAGE=PCLTYPELIST\n");
	 break;
      case 3:
	 if(verbose) printf("Sending @PJL DEFAULT TESTPAGE=PSTYPEFACELIST\n"); 
	 fprintf(pipe,"@PJL DEFAULT TESTPAGE=PSTYPEFACELIST\n");
	 break;
      case 4:
	 /* In PJL spec, but the 2100 doesn't respond */
	 if(verbose) printf("Sending @PJL DEFAULT TESTPAGE=PSCONFIGPAGE\n"); 
	 fprintf(pipe,"@PJL DEFAULT TESTPAGE=PSCONFIGPAGE\n");
	 break;
      case 5:
	 /* In PJL spec, but the 2100 doesn't respond */
	 if(verbose) printf("Sending @PJL DEFAULT TESTPAGE=PSDEMOPAGE\n"); 
	 fprintf(pipe,"@PJL DEFAULT TESTPAGE=PSDEMOPAGE\n");
	 break;
      default:
	 break;
   }
   /* Tell the printer that no more commands are coming... */
   fwrite(escape_sequence,3,1,pipe); 
   fprintf(pipe,"12345X\n");
   
   pclose(pipe);

   return;
}


gint
delete_event                           (GtkButton       *button,
                                        gpointer         user_data)
{
 gtk_main_quit();
 return(FALSE);
}


void
set_resolution                         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  printer_data.resolution = atoi(user_data);

  return;

}


void
set_to_default                         (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET(button),"darkcourier_radio_off")), TRUE); 

 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET(button),"economode_radio_off")), TRUE); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET(button),"resolution_radio_1200")), TRUE); 

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET(button),"density_radio_3")), TRUE); 
 
 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET(button),"traylock_one")), FALSE); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET(button),"traylock_two")), FALSE); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET(button),"traylock_three")), FALSE); 

  return;
}


void
set_economode                          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
   GtkWidget* tempwidget;
   static gint change=1; /* 1 if resolution button should be changed */

   tempwidget = lookup_widget (GTK_WIDGET(togglebutton),
				  "resolution_radio_1200");

   if (printer_data.economode == 1)
   {
      change = 0;
      gtk_widget_show (tempwidget);
   }

   if (printer_data.economode == 0)
   {
      change = 1;
   }

   printer_data.economode = atoi(user_data); 
   
   if ( (printer_data.economode == 1) && (change ==1) )
   {
      /* economode is on, make sure resolution is not 1200 */
      if (GTK_TOGGLE_BUTTON (tempwidget)->active)
      {
	 /* resolution 1200 was selected... choose 600 */
	 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (
	    lookup_widget (GTK_WIDGET(togglebutton),"resolution_radio_600")
	    ), TRUE); 
      }
      gtk_widget_hide (tempwidget);
   }
  return;
}


void
set_darkcourier                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  printer_data.darkcourier = atoi(user_data); 

  return;
}

void
set_density                            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  printer_data.density = atoi(user_data);

  return;
}


void
set_testpage                           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  testpage_num = atoi(user_data);

  return;
}

void
set_traylock                           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
 int selected_tray = atoi(user_data);
  
  if (GTK_TOGGLE_BUTTON (togglebutton)->active)
    {
      /* check box was "checked" */
      switch(selected_tray)
	{
	case 1:
	  printer_data.one_lock = 1;
	  break;
	case 2:
	  printer_data.two_lock = 1;
	  break;
	case 3:
	  printer_data.three_lock = 1;
	  break;
	default:
	  g_print ("Invalid Value (%d) passed to set_traylock\n", selected_tray);
	  break;
	}
    } else {
      /* checkbox was "unchecked" */
       switch(selected_tray)
	{
	case 1:
	  printer_data.one_lock = 0;
	  break;
	case 2:
	  printer_data.two_lock = 0;
	  break;
	case 3:
	  printer_data.three_lock = 0;
	  break;
	default:
	  g_print ("Invalid Value (%d) passed to set_traylock\n", selected_tray);
	  break;
	}
    }

  return;
}


void
get_printer_name                       (GtkEditable     *editable,
                                        gpointer         user_data)
{
  char *selection;

  selection = gtk_entry_get_text (GTK_ENTRY(editable));

  printer_data.name = selection;

  return;

}

GtkWidget *about_dialog;

void
display_about_box                      (GtkButton       *button,
                                        gpointer         user_data)
{

  about_dialog = create_about_dialog();

  gtk_widget_show (about_dialog);
  return;

}


void
destroy_about_dialog                   (GtkButton       *button,
                                        gpointer         user_data)
{

  gtk_widget_destroy (about_dialog);

  return;
}




/* returns 1 if *name has a length less than max_length, 0 otherwise */
int check_length_printername(char *name, int max_length){
  
  if(strlen(name) > max_length){
    return 0;
  }
  else{
    return 1;
  }
}



