/*

g2100config - main.c

Copyright (C) 2000 Bob Duke

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "struct.h"
#include "interface.h"
#include "support.h"

/* defined in struct.h callbacks.c */
extern struct printer_data printer_data;

/* defined in callbacks.c */
extern gint verbose;

int
main (int argc, char *argv[])
{
  GtkWidget *g2100config;
  int opt = 0;
  /* Parse Command Line */
  while((opt = getopt(argc,argv,"vP:"))!=-1){
    switch(opt){  
    case 'P':
      printer_data.name = optarg;
      break;
    case 'v':
      verbose = 1;
      g_print ("You have selected verbose mode... good choice\n");
      break;
    default:
      break;
    }
  }

  /****************************/
  /* Getting the printer name */
  /****************************/
  if(printer_data.name == NULL)
  {
     g_print ("Printer Name not passed with -P option.. looking at PRINTER env variable...\n");
    printer_data.name = getenv("PRINTER");
    if (printer_data.name != NULL)
    {
       g_print ("environment variable found, PRINTER=%s\n", printer_data.name);
    }
  }

  /* If printer_data.name is still NULL, we're out of luck */
  if(printer_data.name == NULL){
    printf("No PRINTER environment variable... you'll have to depend on the GUI selections\n");
    g_print ("To set your PRINTER env variable:\n");
    g_print ("     $ export PRINTER=printername\n");
    g_print ("where the printername is the name of the lj2100 in /etc/printcap\n\n");
    //printer_data.name = "lp";
  }
  
  /*********************************/
  /* End Getting the printer name */
  /********************************/
  
#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  g2100config = create_g2100config ();
  gtk_widget_show (g2100config);

  gtk_main ();
  return 0;
}
